import os
import re
import sys

"""
This script modifies vehicle .xtbl files (for Saints Row: The Third) to remove angular damping.
It only processes files with certain prefixes (designated by 'prefixes') and which have the
suffix '_veh'. You can edit 'prefixes' below to change what types of vehicle the script
applies to - hopefully it's pretty self explanatory. Put a comma after every item besides the
last one, and make sure every item is enclosed in matching quote marks.

Needless to say you'll need Python 3 installed to use this!

To use the script, set up the following conditions:
- put the script in an empty folder on its own.
- create a folder called 'out' to receive finished files.
- put all the files to be processed in the same folder as the script (not the 'out' folder).

The 'out' folder doesn't strictly need to be empty, but the script will overwrite any existing
files without asking for confirmation.

When you run the script, it will report what it does for each file. By default if it does not
see anything to modify in a particular file, it won't write it to 'out'. If you would like it
to do that anyway, you can add the command line option -w. This effectively copies those files
so you can simply install the contents of the 'out' folder.

This script is quite rough and ready, which means it will crash if it comes across anything
invalid. Please make sure your files are valid vehicle tables, and that your folder structure
is set up correctly.

Released by MegaJohnny (megajohnny@hotmail.co.uk) under WTFPL.
Have fun and enjoy the new handling!
"""

prefixes = [
'car',
'sp',
'truck',
'van',
'minivan',
'bike',
'suv',
'wagon' #only keystone has this AFAIK
]

suffix = '_veh.xtbl'

#each tag is the XML tag with a sequence of digits and/or periods inside it
normalTag = re.compile('<Normal_Spin_Damping>(-?[\d\.]+)</Normal_Spin_Damping>')
collisionTag = re.compile('<Collision_Spin_Damping>(-?[\d\.]+)</Collision_Spin_Damping>')

forceModify = False

if len(sys.argv) == 2 and sys.argv[1] == '-w':
    forceModify = True

#given a file name, this function sets the spin dampings correctly and (if applicable) writes to out
angularTag = '<Angular_Damping>'
def RemoveDamping(fname):
    contents = ''
    modified = False
    with open(fname, 'r') as f:
        contents = f.read()
        
        #correct/create normal spin damping tag
        normalMatch = normalTag.search(contents)
        if normalMatch:
            if normalMatch.group(1).strip() != '0':
                print(' - normal damping is non-zero, rectifying')
                valStart = normalMatch.start(1)
                valEnd = normalMatch.end(1)
                contents = contents[:valStart] + '0' + contents[valEnd:]
                modified = True
        else:
            print(' - normal damping doesn\'t exist, rectifying')
            dampPos = contents.find(angularTag)
            insertPos = dampPos + len(angularTag)
            
            contents = contents[:insertPos] + '<Normal_Spin_Damping>0</Normal_Spin_Damping>' + contents[insertPos:]
            modified = True
            
        #correct/create normal spin damping tag
        collisionMatch = collisionTag.search(contents)
        if collisionMatch:
            if collisionMatch.group(1).strip() != '0':
                print(' - collision damping is non-zero, rectifying')
                valStart = collisionMatch.start(1)
                valEnd = collisionMatch.end(1)
                contents = contents[:valStart] + '0' + contents[valEnd:]
                modified = True
        else:
            print(' - collision damping doesn\'t exist, rectifying')
            dampPos = contents.find(angularTag)
            insertPos = dampPos + len(angularTag)
            
            contents = contents[:insertPos] + '<collision_Spin_Damping>0</collision_Spin_Damping>' + contents[insertPos:]
            modified = True
            
    if not modified and forceModify:
        print(' - nothing to do but writing to output anyway')
        modified = True
                
    if modified:
        with open(os.path.join('out',fname), 'w') as f:
            f.write(contents)
    else:
        print(' - nothing to do')
    print()

#returns true iff the filename before the first _ is in 'prefixes'
def ApplicableSuffix(f):
    return f[:f.find('_')] in prefixes

#this is the main entry point
for f in os.listdir():
    if f.endswith(suffix) and ApplicableSuffix(f):
        print('Processing {0}:'.format(f))
        RemoveDamping(f)
    else:
        pass
        #print('Ignoring {0}'.format(f))